"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging

from azure.core.exceptions import ResourceNotFoundError

from hcs_core.sglib.client_util import wait_for_res_status
from hcs_ext_azure.provider._az_facade import resource_management_client as client
from hcs_ext_azure.provider._az_facade import storage_management_client as sclient
from azure.storage.fileshare import ShareClient, ShareFileClient
from hcs_core.util import hcs_constants

log = logging.getLogger(__file__)


def fs_copy(data: dict, state: dict):
    _copy_metadata = []
    _source_files = data["sourceFiles"]
    _file_path = data["filePath"]
    _dest_client = _get_des_client(data["resGroup"], data["destinationShareName"], data["storageAccountName"])
    for _blob in _source_files:
        res = check_copy_done(state, _blob)
        _f_name = _blob.split(hcs_constants.FORWARD_SLASH)[-1]
        _dest_file = _dest_client.get_file_client(_file_path + _f_name)
        if not res:
            _dest_file.start_copy_from_url(_blob)
            _result = _wait_for_fs_copy_completion(data["resGroup"], _blob, _dest_file)
        _copy_metadata.append(_file_properties_to_dict(_dest_file.get_file_properties()))
    return _copy_metadata


def check_copy_done(state: dict, _blob):
    result = False
    if state:
        for _rec in state:
            if _rec.get("fsource") and _rec.get("fsource") == _blob and _rec.get("status") == "success":
                result = True
                break
            else:
                result = False
    return result


def _get_sa_info(res_grp_name: str, acc_name: str):
    ret_val = dict()
    for i in client().resources.list_by_resource_group(res_grp_name):
        if i.type == hcs_constants.MS_STORAGE_ACCOUNTS:
            storage_keys = sclient().storage_accounts.list_keys(res_grp_name, acc_name)
            storage_keys = {v.key_name: v.value for v in storage_keys.keys}
            ret_val["storage_key"] = storage_keys["key1"]
    return ret_val


def _get_des_client(res_grp: str, des_share_name: str, storage_acc_name: str):
    _dest_client = None
    try:
        ret_val = _get_sa_info(res_grp, storage_acc_name)
        if ret_val:
            acc_key = ret_val.get("storage_key")
            des_con_str = (
                hcs_constants.CONN_STR_1
                + storage_acc_name
                + hcs_constants.CONN_STR_2
                + acc_key
                + hcs_constants.CONN_STR_3
            )
            _dest_client = ShareClient.from_connection_string(des_con_str, des_share_name)
        else:
            raise Exception("Unknown error occurred while copying source files to storage account")
    except ResourceNotFoundError as e:
        pass
    except Exception as e:
        raise Exception("Exception while creating destination client.") from e
    return _dest_client


def _file_properties_to_dict(_file_properties):
    return {
        "name": _file_properties.name,
        "path": _file_properties.path,
        "share": _file_properties.share,
        "size": _file_properties.size,
        "status": _file_properties.copy.status,
        "fsource": _file_properties.copy.source,
    }


def _wait_for_fs_copy_completion(res_group: str, _blob: str, _dest_file: ShareFileClient, timeout: str = "10m"):
    name = "fs-copy-completion-/" + res_group

    def fn_get():
        return _check_fs_copy_done(_blob, _dest_file)

    status_map = {
        "ready": hcs_constants.COMPLETE,
        "error": hcs_constants.ERROR,
        "transition": hcs_constants.FS_COPY_TRANSITION_STATUS,
    }
    return wait_for_res_status(
        resource_name=name, fn_get=fn_get, get_status=hcs_constants.STATUS, status_map=status_map, timeout=timeout
    )


def _check_fs_copy_done(_blob: str, _dest_file: ShareFileClient):
    _result = dict()
    _result["status"] = hcs_constants.START
    try:
        _f_prop = _dest_file.get_file_properties()
        _f_copy_status = _f_prop["copy"]["status"]
        if _f_prop and _f_copy_status == hcs_constants.SUCCESS:
            _result["status"] = hcs_constants.COMPLETE
        else:
            _result["status"] = hcs_constants.INCOMPLETE
    except Exception as e:
        _result["status"] = hcs_constants.ERROR
        raise
    return _result


def fs_refresh(data: dict, state: dict):
    if _is_fs_res_grp_exists(data["resGroup"]):
        _file_list = _get_sa_fs(data)
        if state:
            for _rec in state:
                if _rec.get("name") and _rec.get("name") not in _file_list:
                    _rec["status"] = hcs_constants.START
    return state


def _get_sa_fs(data: dict):
    _file_list = []
    try:
        _full_file_path = data["destinationShareName"] + hcs_constants.FORWARD_SLASH + data["filePath"]
        _dest_client = _get_des_client(data["resGroup"], _full_file_path, data["storageAccountName"])
        if _dest_client:
            _file_list = [file["name"] for file in list(_dest_client.list_directories_and_files())]
    except ResourceNotFoundError as e:
        pass
    return _file_list


def fs_delete(data: dict):
    if _is_fs_res_grp_exists(data["resGroup"]):
        _file_list = _get_sa_fs(data)
        _source_files = data["sourceFiles"]
        _file_path = data["filePath"]
        _dest_client = _get_des_client(data["resGroup"], data["destinationShareName"], data["storageAccountName"])
        for _blob in _source_files:
            _f_name = _blob.split(hcs_constants.FORWARD_SLASH)[-1]
            if _file_list and _f_name in _file_list:
                _dest_file = _dest_client.get_file_client(_file_path + _f_name)
                _dest_file.delete_file()
    return


def _is_fs_res_grp_exists(res_grp_name: str):
    ret_val = False
    try:
        for _i in client().resources.list_by_resource_group(res_grp_name):
            if _i and _i.id and res_grp_name in _i.id:
                ret_val = True
                break
            else:
                ret_val = False
    except ResourceNotFoundError as e:
        ret_val = False
    return ret_val



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging
import yaml
from typing import Callable
from hcs_ext_azure.provider import _az_facade as az
from hcs_cli.service import admin
from hcs_core.ctxp import data_util, CtxpException
from os import path
from . import template, infra

log = logging.getLogger(__file__)


# def prep_az_cli(tenant_request):
#     provider = tenant_request["provider"]
#     providerInstanceId = provider["providerInstanceId"]
#     print("Provider:", providerInstanceId)
#     providerInstance = admin.provider.get("azure", providerInstanceId)
#     subscription_id = providerInstance["providerDetails"]["data"]["subscriptionId"]
#     directory_id = providerInstance["providerDetails"]["data"]["directoryId"]
#     application_id = providerInstance["providerDetails"]["data"]["applicationId"]
#     region = providerInstance["providerDetails"]["data"]["region"]
#     print("Subscription:", subscription_id)
#     print("Directory:", directory_id)
#     print("ApplicationId:", application_id)
#     print("Region:", region)
#     if application_id != provider["applicationId"]:
#         log.warning("Configured application ID for CLI does not match application ID for provider.")

#     az.login(provider["applicationId"], provider["applicationKey"], directory_id)
#     az.set_subscription(subscription_id)


def prepare_plan_file(deployment_id: str, blueprint_id: str, fn_collect_info: Callable):
    suffix = ".blueprint.yml"

    if blueprint_id.endswith(suffix):
        blueprint_id = blueprint_id[: -len(suffix)]

    input_data = template.get(blueprint_id + ".var.yml")
    file_name = _get_file_name(deployment_id)

    # Apply previous input, if any
    prev = data_util.load_data_file(file_name)
    if prev:
        prev_vars = prev.get("var")
        if prev_vars:
            data_util.deep_apply_default(input_data["var"], prev_vars)

    # Add default from shared infra config, if anything missing
    data_util.deep_apply_default(input_data, infra.get())

    # Enforce key values
    input_data["deploymentId"] = deployment_id
    input_data["var"]["orgId"] = _get_org_id()

    def create_file(success):
        # Combine and save
        blueprint_file = blueprint_id + ".blueprint.yml"
        blueprint = template.get(blueprint_file)
        text = "\n".join(
            [
                yaml.safe_dump(input_data, sort_keys=False),
                "",
                "# ----------------------------------",
                "# Blueprint: " + blueprint_file,
                "",
                yaml.safe_dump(blueprint, sort_keys=False),
            ]
        )
        with open(file_name, "w") as file:
            file.write(text)

        if success:
            print("Plan saved as file: " + file_name)
            print(f"To view the plan:  'hcs plan graph -f {file_name}'")
            print(f"To apply the plan: 'hcs plan apply -f {file_name}'")

    success = False
    try:
        # Collect input from user
        fn_collect_info(input_data)
        success = True
    except Exception:
        raise
    finally:
        create_file(success)


def prepare_order_definition(type: str, template_id: str, fn_collect_info: Callable, order_def={}):
    suffix = ".var.yml"

    if template_id.endswith(suffix):
        template_id = template_id[: -len(suffix)]

    input_data = template.get(template_id + suffix)
    input_data["orderType"] = type
    if order_def:
        input_data["var"] = order_def
    else:
        input_data["var"]["orgId"] = _get_org_id()

    try:
        # Collect input from user
        fn_collect_info(input_data)
    except Exception:
        raise

    return input_data


def _get_file_name(deployment_id: str, suffix=".plan.yml") -> str:
    return deployment_id + suffix


def _get_org_id():
    from hcs_core.sglib import auth

    auth_info = auth.details(get_org_details=False)
    if not auth_info:
        raise CtxpException("Not authenticated. See 'hcs login --help'.")
    return auth_info["org"].id



import ipaddress
from typing import Tuple
from ipaddress import ip_network


def _formalize(cidrs: list[str]):
    def to_network(n):
        return ip_network(n)

    cidrs2 = map(to_network, cidrs)
    return list(ipaddress.collapse_addresses(cidrs2))


def _has_overlap(network, list_of_networks):
    for n in list_of_networks:
        if network.overlaps(n):
            return True


def _is_subnet(network, list_of_networks):
    for n in list_of_networks:
        if network.subnet_of(n):
            return True


def find_available_cidr_24(total_cidrs: list[str], used_cidrs: list[str], configured_cidrs: list[str]):
    total_cidrs = _formalize(total_cidrs)
    used_cidrs = _formalize(used_cidrs)
    configured_cidrs = _formalize(configured_cidrs)

    for t in configured_cidrs:
        for space in t.subnets(new_prefix=24):
            if not _is_subnet(space, total_cidrs):
                continue
            if _has_overlap(space, used_cidrs):
                continue
            return str(space)


def subnets_of(input: list[str], limit: list[str]) -> Tuple[bool, str]:
    input = _formalize(input)
    limit = _formalize(limit)
    for i in input:
        if not _is_subnet(i, limit):
            return False, str(i)
    return True, None


def overlaps(a: list[str], b: list[str]) -> Tuple[bool, str]:
    a = _formalize(a)
    b = _formalize(b)

    for i in a:
        if _has_overlap(i, b):
            return True, i
    return False, None



import hcs_core.ctxp.data_util as data_util
from os import path
from hcs_core.plan import PlanException
import logging

log = logging.getLogger(__name__)

_template_dir = path.abspath(path.join(path.dirname(__file__), "templates"))


def get(name: str, raise_on_not_found: bool = True):
    file_name = path.join(_template_dir, name)
    ret = data_util.load_data_file(file_name)

    if raise_on_not_found and not ret:
        log.error("Template path: " + file_name)
        raise PlanException("Template not found: " + name)
    return ret



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import yaml
from ulid import ULID
import hcs_cli.service.admin as admin
from hcs_core.plan import context
import logging

log = logging.getLogger(__name__)


def process(data: dict, state: dict) -> dict:
    users = data["users"]
    entitlementId = data["entitlementId"]
    domainName = data["domainName"]
    stackUrl = data["stackUrl"]

    launch_items = []

    for u in users:
        horizonId = str(ULID())
        login_hint = u["userPrincipalName"]
        launch_items.append(
            {
                "userPrincipalName": u["userPrincipalName"],
                "password": u["password"],
                "launchUrl": f"{stackUrl}/appblast/webclient/?horizonId={horizonId}&entitlementId={entitlementId}&domainName={domainName}&action=start-session&login_hint={login_hint}#/desktop",
            }
        )

    uag = admin.helper.get_uags_by_edge(data["edgeId"], data["orgId"])[0]

    fqdn = uag["fqdn"]
    ip = uag["loadBalancer"]["ipAddress"]
    ret = {"uag": {"fqdn": fqdn, "ip": ip}, "logins": launch_items}

    file_name = f'{context.get("deploymentId")}-secret.yml'
    with open(file_name, "w") as file:
        yaml.dump(ret, file, sort_keys=False)

    log.info(f"Manual operation needed: update DNS {fqdn} -> {ip}.")
    log.info("Login secrets: %s", file_name)
    return ret


def destroy(data: dict, state: dict, force: bool) -> dict:
    return


def eta(action: str, data: dict, state: dict):
    return "1m"



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging
from hcs_core.ctxp import choose
from hcs_cli.service import ims, av

log = logging.getLogger(__file__)


def configure_desktops(var: dict):
    org_id = var["orgId"]

    def _select_desktop_type():
        types = ["FLOATING", "DEDICATED", "MULTI_SESSION"]
        template_type = choose("Desktop type:", types)
        var["template"]["type"] = template_type
        return template_type

    def _select_image_and_vm_sku():
        multi_session = False
        if var["template"]["type"] == "MULTI_SESSION":
            multi_session = True

        images = ims.helper.get_images_by_provider_instance_with_asset_details(var["providerInstanceId"], org_id)
        images = [i for i in images if i["multiSession"] == multi_session]
        fn_get_text = lambda d: f"{d['name']}: {d['os']}"
        prev_selected_image = None
        if var["image"]["streamId"]:
            for i in images:
                if i["id"] == var["desktop"]["streamId"]:
                    prev_selected_image = i
                    break
        selected_image = choose("Select image:", images, fn_get_text, selected=prev_selected_image)
        var["image"]["streamId"] = selected_image["id"]
        var["image"]["name"] = selected_image["name"]
        var["image"]["os"] = selected_image["os"]

        fn_get_text = lambda m: f"{m['name']}"
        selected_marker = choose("Select marker:", selected_image["markers"], fn_get_text)
        var["image"]["markerId"] = selected_marker["id"]
        var["image"]["markerName"] = selected_marker["name"]

        image_asset_details = selected_image["_assetDetails"]["data"]
        var["image"]["sku"] = image_asset_details["vmSize"]
        var["image"]["gen"] = "Gen2" if image_asset_details["generationType"] == "V2" else "Gen1"

    _select_desktop_type()
    _select_image_and_vm_sku()


def configure_apps(var: dict):
    org_id = var["orgId"]

    def _select_applications():
        app_info_list = []
        av_list = {}
        fn_get_text = lambda d: f"{d['name']}: {d['description']}"
        while _add_application() == "1":
            av_list = av.app.list_apps(org_id) if not av_list and not app_info_list else av_list
            if not av_list:
                log.info("No applications found. Skipping app configurations")

            _app_info = {}
            _selected_app = choose("Select application:", av_list, fn_get_text, selected=None, select_by_default=False)
            _app_info["applicationId"] = _selected_app["id"]
            _app_info["name"] = _selected_app["name"]

            _app_versions = _selected_app.get("appVersions")
            if not _app_versions:
                log.info(
                    "No versions found for application '%s'. Skipping further configuration for this application",
                    _selected_app["name"],
                )
                av_list = [app for app in av_list if app.get("id", "") != _app_info.get("applicationId")]
            else:
                latest_app_version = max(_app_versions, key=lambda k: k["updatedAt"])
                _selected_version = choose(
                    "Select application version:",
                    _app_versions,
                    fn_get_text,
                    selected=latest_app_version,
                    select_by_default=True,
                )
                _app_info["appVersionId"] = _selected_version["id"]
                _app_info["version"] = _selected_version["version"]

            if "appVersionId" in _app_info and "applicationId" in _app_info and _app_info not in app_info_list:
                app_info_list.append(_app_info)
                av_list = [app for app in av_list if app.get("id", "") != _app_info.get("applicationId")]

            if not av_list:
                log.info("All existing applications configured successfully")
                break

        var["application"]["info"] = app_info_list

    def _add_application():
        options = ["Yes", "No"]
        input_option = choose("Do you want to add applications to order:", options)
        if input_option == "No":
            return "0"
        return "1"

    _select_applications()



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging
from hcs_cli.service import admin, auth
from hcs_ext_azure.provider import _az_facade as az
from hcs_core.plan import PluginException
from .cidr_util import find_available_cidr_24

log = logging.getLogger(__name__)


def process(data: dict, state: dict) -> dict:
    org_id = data["orgId"]
    edge = admin.edge.get(data["edgeId"], org_id)
    if not edge:
        raise PluginException("Edge not found: " + data["edgeId"])
    provider_id = edge["providerInstanceId"]
    # log.info('Provider: %s', provider_id)
    provider = admin.provider.get("azure", provider_id, org_id)
    if not provider:
        raise PluginException("Provider not found: " + provider_id)
    providerData = provider["providerDetails"]["data"]
    subscription_id = providerData["subscriptionId"]
    # directory_id = providerData['directoryId']
    # application_id = providerData['applicationId']
    region = providerData["region"]
    # log.info('Subscription: %s', subscription_id)
    # log.info('Directory: %s', directory_id)
    # log.info('ApplicationId: %s', application_id)
    # log.info('Region: %s', region)

    number_of_users = len(data["userEmails"])
    is_multi_session = "MULTI_SESSION" == data["order"]["template"]["type"]

    uag_deployments = admin.helper.list_resources_by_provider("uag-deployments", provider_id, limit=1, org_id=org_id)
    if not uag_deployments:
        raise Exception("No UAG deployment found.")
    uag_deployment_id = uag_deployments[0]["id"]

    edge_deployments = admin.helper.list_resources_by_provider("edge-deployments", provider_id, limit=1, org_id=org_id)
    if not edge_deployments:
        raise Exception("No UAG deployment found.")
    edge_deployment_id = edge_deployments[0]["id"]

    search = f"name $eq {data['order']['image']['sku']}"
    vm_skus = admin.azure_infra.get_compute_vm_skus(
        provider_instance_id=provider_id, search=search, limit=1, org_id=org_id
    )
    if not vm_skus:
        raise Exception("No VM SKUs found.")

    if is_multi_session:
        sessions_per_vm = 10
        total_vms = int((number_of_users + sessions_per_vm - 1) / sessions_per_vm)
    else:
        sessions_per_vm = 1
        total_vms = number_of_users
    template = {
        "total_vms": total_vms,
        "sessions_per_vm": sessions_per_vm,
        "password": _generate_password(),
        "uag_deployment_id": uag_deployment_id,
        "edge_deployment_id": edge_deployment_id,
        "vm_sku": vm_skus[0],
    }

    org_idp_map = auth.admin.get_org_idp_map()
    vnet = az.network.vnet.get(edge["infrastructure"]["managementNetwork"]["data"]["parent"])
    cidr = _calculate_cidr(vnet)
    log.info("Calculated network address: %s", cidr)
    return {
        "location": region,
        "cidr": cidr,
        "vNet": vnet,
        "provider": {"id": provider_id, "subscriptionId": subscription_id},
        "template": template,
        "orgIdpMap": org_idp_map,
    }


def _generate_password():
    from random import choice

    upper_chars = "ABCDEFGHJKLMNPQRSTUVWXY"
    readable_chars = "abcdefghjklmnpqrstuvwxy3456789"
    special_chars = "!@#$%_"
    return "" + choice(upper_chars) + "".join(choice(readable_chars) for i in range(12)) + choice(special_chars)


def _calculate_cidr(vnet):
    vnet_cidrs = vnet["addressSpace"]["addressPrefixes"]
    used_cidrs = []
    for subnet in vnet["subnets"]:
        used_cidrs.append(subnet["addressPrefix"])
    # https://confluence.eng.vmware.com/display/HCS/Titan+Lite+-+BOM#TitanLiteBOM-AddressSpaces
    infra_cidr = None
    for c in vnet_cidrs:
        if not c.endswith(".0/16"):
            continue
        infra_cidr = c
        break
    if not infra_cidr:
        raise PluginException("No /16 CIDR found in vNet.")

    prefix = infra_cidr[: infra_cidr.rindex(".")]
    reserved_cidr = prefix + ".0/24"
    used_cidrs.append(reserved_cidr)
    tenant_cidr = find_available_cidr_24(vnet_cidrs, used_cidrs, [infra_cidr])
    if not tenant_cidr:
        raise PluginException(
            "Unable to find usable subnet address space for the tenant. Consider adding new address space to the config via 'hcs daas infra plan', and/or add new spaces in vnet."
        )
    return tenant_cidr


def destroy(data: dict, state: dict, force: bool):
    return


def eta(action: str, data: dict, state: dict):
    return "1m"



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import re
import ipaddress
import hashlib
import logging
from hcs_cli.service import admin
from hcs_ext_azure.provider import _az_facade as az
from hcs_core.plan import PlanException, context

log = logging.getLogger(__name__)


def process(data: dict, state: dict) -> dict:
    org_id = data["orgId"]
    provider = data["provider"]
    provider_id = provider["id"]
    if provider_id:
        log.info("Reusing provider: %s", provider_id)
        providerInstance = admin.provider.get("azure", provider_id, org_id)
        if providerInstance:
            providerData = providerInstance["providerDetails"]["data"]
            region = providerData["region"]
        else:
            raise PlanException("Provider not found: " + provider_id)
    else:
        region = data["provider"]["region"]
        provider_id = state["output"]["myProvider"]["id"]

    # identify subnets
    subnet_map = _prepare_subnet_map(data["network"]["vNetId"])

    edge = {"managementNetwork": subnet_map[data["network"]["managementNetworkName"]]}
    uag = {
        "managementNetwork": subnet_map[data["network"]["managementNetworkName"]],
        "dmzNetwork": subnet_map[data["network"]["dmzNetworkName"]],
        "desktopNetwork": subnet_map[data["network"]["desktopNetworkName"]],
    }
    image = {
        "desktopSubnetId": uag["desktopNetwork"]["id"],
        "desktopPassword": [c for c in data["desktop"]["adminPassword"]],
        "multiSessionImageName": _generate_image_name("m"),
        "desktopImageName": _generate_image_name("d"),
    }

    return {
        "location": region,
        "providerInstanceId": provider_id,
        "edge": edge,
        "uag": uag,
        "image": image,
    }


def _prepare_subnet_map(vnet_id):
    # /subscriptions/a2ef2de8-f2b5-43da-bf68-2b182dd5f928/resourceGroups/horizonv2-sg-dev/providers/Microsoft.Network/virtualNetworks/westus2-vnet-1
    pattern = r"/subscriptions/.+?/resourceGroups/(.+?)/providers/Microsoft.Network/virtualNetworks/(.+)"
    matcher = re.search(pattern, vnet_id)
    if not matcher:
        raise PlanException("Fail parsing vNet ID: " + vnet_id)
    rg = matcher.group(1)
    vnet_name = matcher.group(2)
    subnets = az.network.subnet.list(rg, vnet_name)

    ret = {}
    for subnet in subnets:
        subnet_name = subnet["name"]
        cidr = subnet["addressPrefix"]
        addrs = ipaddress.ip_network(cidr).num_addresses - 4
        ret[subnet_name] = _to_network(vnet_id, subnet_name, addrs, cidr)
    return ret


def _to_network(vnet_id, subnet_name, available_ip_addresses, cidr):
    return {
        "kind": "subnets",
        "id": f"{vnet_id}/subnets/{subnet_name}",
        "data": {"parent": vnet_id, "name": subnet_name, "availableIpAddresses": available_ip_addresses, "cidr": cidr},
    }


def _generate_image_name(prefix: str) -> str:
    ret = prefix + "-" + context.get("deploymentId")
    # 11 chars max
    return _to_limited_string(ret, 11)


def _to_limited_string(text: str, limit: int) -> str:
    l = len(text)
    if l <= limit:
        return text
    first_part = text[: limit - 4]
    second_part = text[limit - 4 :]

    sha256_hash = hashlib.sha256(second_part.encode()).hexdigest()
    truncated_hash = sha256_hash[:3]
    return first_part + "-" + truncated_hash


def destroy(data: dict, state: dict, force: bool):
    return


def eta(action: str, data: dict, state: dict):
    return "1m"



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from hcs_core.ctxp import context


_config_name = "daas-infra"


def get():
    return _get_config()


def set(data: dict):
    return context.set(_config_name, data)


def file():
    data = _get_config()
    context.set(_config_name, data)
    return context.file(_config_name)


def _get_config():
    data = context.get(_config_name, default={})
    return _with_default(data, _config_template)


def _with_default(target: dict, default: dict) -> dict:
    ret = dict(default)
    ret.update(target)
    return ret


_config_template = {
    "provider": {"id": ""},
    "network": {"vNetId": "", "tenantCIDRs": []},
    "desktop": {"markerId": "", "streamId": "", "templateType": "", "vmSkuName": ""},
}



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from hcs_core.ctxp import context
from datetime import datetime, timezone

_config_name = "daas-order"


def get():
    return _get_config()


def add(data: dict):
    current_orders = _get_config()
    current_orders.update(data)
    return context.set(_config_name, current_orders)


def remove(order_type: str):
    current_orders = _get_config()

    if not order_type in current_orders.keys():
        return

    removed_order = current_orders.pop(order_type)
    removed_order["deleted_at"] = str(datetime.now(timezone.utc))
    current_orders["deleted"].append({order_type: removed_order})
    return context.set(_config_name, current_orders)


def file():
    data = _get_config()
    context.set(_config_name, data)
    return context.file(_config_name)


def _get_config():
    return context.get(_config_name, default={"deleted": []})



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging

from hcs_cli.service import ims, av
from . import template, order

log = logging.getLogger(__name__)


def process(data: dict, state: dict) -> dict:
    org_id = data["orgId"]
    provider_id = _get_provider_id(data, state)

    images = ims.helper.get_images_by_provider_instance_with_asset_details(provider_id, org_id)

    if not images:
        raise Exception(
            "Failed to create default orders with infra images for org " + org_id + " provider_id " + provider_id
        )

    default_orders = [
        _create_default_order(data, state, images, multi_session=True),
        _create_default_order(data, state, images, multi_session=False),
    ]
    return default_orders


def _create_default_order(data: dict, state: dict, images: dict, multi_session: str):
    template_type = "MULTI_SESSION" if multi_session else "FLOATING"
    selected_image = None
    for image in images:
        if image["multiSession"] == multi_session:
            selected_image = image
            break
    if not selected_image:
        raise Exception("Failed to create default order for template: " + template_type)

    order_data = template.get("v1/tenant-order.var.yml")["var"]
    order_data["orgId"] = data["orgId"]
    order_data["providerInstanceId"] = _get_provider_id(data, state)
    order_data["edgeDeploymentId"] = _get_edge_id(data, state)
    order_data["application"]["info"] = []
    order_data["template"]["type"] = template_type
    order_data["image"]["streamId"] = selected_image["id"]
    order_data["image"]["name"] = selected_image["name"]
    order_data["image"]["os"] = selected_image["os"]
    order_data["application"]["info"] = _get_app_info_list(data["orgId"])

    if selected_image["markers"]:
        order_data["image"]["markerId"] = selected_image["markers"][0]["id"]
        order_data["image"]["markerName"] = selected_image["markers"][0]["name"]

    if selected_image["_assetDetails"]:
        order_data["image"]["sku"] = selected_image["_assetDetails"]["data"]["vmSize"]
        order_data["image"]["gen"] = (
            "Gen2" if selected_image["_assetDetails"]["data"]["generationType"] == "V2" else "Gen1"
        )

    order_type = state["deploymentId"] + "-default-" + template_type
    order.add({order_type: order_data})
    return order_type


def _get_provider_id(data: dict, state: dict):
    if data["provider"] and "id" in data["provider"] and data["provider"]["id"]:
        return data["provider"]["id"]

    return state["output"]["myProvider"]["id"]


def _get_edge_id(data: dict, state: dict):
    if data["edge"] and "id" in data["edge"] and data["edge"]["id"]:
        return data["edge"]["id"]

    return state["output"]["myEdge"]["id"]


def _get_app_info_list(org_id: str):
    app_info_list = []
    av_list = av.app.list_apps(org_id)
    if not av_list:
        return app_info_list

    for app in av_list:
        _app_info = {"applicationId": app["id"], "name": app["name"]}

        _app_versions = app.get("appVersions")
        if not _app_versions:
            continue
        else:
            latest_app_version = max(_app_versions, key=lambda k: k["updatedAt"])
            if latest_app_version:
                _app_info["appVersionId"] = latest_app_version["id"]
                _app_info["version"] = latest_app_version["version"]

        if "appVersionId" in _app_info and "applicationId" in _app_info and _app_info not in app_info_list:
            app_info_list.append(_app_info)

    return app_info_list


def destroy(data: dict, state: dict, force: bool):
    if state:
        for order_type in state:
            order.remove(order_type)


def eta(action: str, data: dict, state: dict):
    return "1m"
